-----------------------------------
    Population: GPi_0_0
-----------------------------------
  N=100
  C=0.500000 nF
  Membrane time constant=20.000000 ms
  Resting potential=-70.000000 mV
  Reset potential=-55.000000 mV
  Threshold =-50.000000 mV
  RestPot_ca =-85.000000 mV
  Alpha_ca =0.500000 mV
  Tau_ca =80.000000 mV
  Eff_ca =0.000000 mV
  tauhm=20.000000 ms
  tauhp=100.000000 ms
  V_h=-60.000000 mV
  V_T=120.000000 mV
  g_T=0.000000 mS
Receptor 0: GABA (Mg block: 0)
  Tau=5.000000 ms
  Reversal potential=-70.000000 mV
  Ext frequency=0.000000 Hz
  Ext frequency SD=0.000000 Hz
  Ext efficacy=0.000000 nS
  Ext connections=0.000000
Receptor 1: AMPA (Mg block: 0)
  Tau=2.000000 ms
  Reversal potential=0.000000 mV
  Ext frequency=0.800000 Hz
  Ext frequency SD=0.000000 Hz
  Ext efficacy=5.900000 nS
  Ext connections=800.000000
Receptor 2: NMDA (Mg block: 1)
  Tau=100.000000 ms
  Reversal potential=0.000000 mV
  Ext frequency=0.000000 Hz
  Ext frequency SD=0.000000 Hz
  Ext efficacy=0.000000 nS
  Ext connections=0.000000
Synapses targeting population: Th_0_0 (14)
   Target receptor code=0
  Connectivity=1.000000
  Mean efficacy=0.142375
EndPopulation
